@@ -59,10 +59,13 @@
 
 static const ZeroTier::MulticastGroup _blindWildcardMulticastGroup(ZeroTier::MAC(0xff),0);
 
+#define MACOS_FETH_MAX_MTU_SYSCTL "net.link.fake.max_mtu"
+
 namespace ZeroTier {
 
 static Mutex globalTapCreateLock;
 static bool globalTapInitialized = false;
+static bool fethMaxMtuAdjusted = false;
 
 MacEthernetTap::MacEthernetTap(
 	const char *homePath,
@@ -102,6 +105,14 @@ MacEthernetTap::MacEthernetTap(
 
 	Mutex::Lock _gl(globalTapCreateLock); // only make one at a time
 
+	if (!fethMaxMtuAdjusted) {
+		fethMaxMtuAdjusted = true;
+		int old_mtu = 0;
+		size_t old_mtu_len = sizeof(old_mtu);
+		int mtu = 10000;
+		sysctlbyname(MACOS_FETH_MAX_MTU_SYSCTL, &old_mtu, &old_mtu_len, &mtu, sizeof(mtu));
+	}
+
 	// Destroy all feth devices on first tap start in case ZeroTier did not exit cleanly last time.
 	// We leave interfaces less than feth100 alone in case something else is messing with feth devices.
 	if (!globalTapInitialized) {
@@ -198,8 +209,30 @@ MacEthernetTap::MacEthernetTap(
 		::_exit(-1);
 	} else {
 		_agentPid = apid;
+
+		// Wait up to 10 seconds for the subprocess to actually create the device. This prevents
+		// things like routes from being created before the device exists.
+		for(int waitLoops=0;;++waitLoops) {
+			struct ifaddrs *ifa = (struct ifaddrs *)0;
+			if (!getifaddrs(&ifa)) {
+				struct ifaddrs *p = ifa;
+				while (p) {
+					if ((p->ifa_name)&&(!strcmp(devstr, p->ifa_name))) {
+						waitLoops = -1;
+						break;
+					}
+					p = p->ifa_next;
+				}
+				freeifaddrs(ifa);
+			}
+			if (waitLoops == -1) {
+				break;
+			} else if (waitLoops >= 100) { // 10 seconds
+				throw std::runtime_error("feth device creation timed out");
+			}
+			Thread::sleep(100);
+		}
 	}
-	Thread::sleep(100); // this causes them to come up in a more user-friendly order on launch
 
 	_thread = Thread::start(this);
 }
